declare module "utils/arraydiff" {
    const _default: (one: any[], two: any[]) => any[];
    export default _default;
}
declare module "object" {
    import TypeBase from "base";
    import Validator from "validator";
    import { LangList } from "types/index";
    export const messages: LangList;
    export interface OptionsIn {
        oneErrorPerKey: boolean;
    }
    /**
     * Object class
     */
    export class TypeObject extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force an object to have only the keys passed in the set
         */
        in(...array: string[]): this;
        in(array: string[]): this;
        in(array: string[], options: OptionsIn): this;
        /**
         * Force an object to not have the keys passed in the set
         *
         * @param  {...string} array
         * @return {this}
         */
        not(...array: string[]): this;
        not(array: string[]): this;
        /**
         * Validate an object with a fully qualified schema
         *
         * @param  {Altheia} schema
         * @param  {boolean} options.returnErrors If true, deep errors while be returned too
         * @return {this}
         */
        schema(schema: Validator, { returnErrors }?: {
            returnErrors?: boolean | undefined;
        }): this;
        /**
         * Force any keys, to be the only one present in the object
         *  (exclusive relationships)
         *  oneIsRequired is false by default
         *
         * @param  {mixed} params
         * @return {this}
         */
        oneOf(...params: (string | boolean)[]): this;
        /**
         * Force all keys to be mutually required. If one is presents, all are required. Pass if none are present.
         *
         * @param  {...string} keys
         * @return {this}
         */
        allOf(...keys: string[]): this;
    }
    const def: {
        Class: typeof TypeObject;
        messages: LangList;
    };
    export default def;
}
declare module "utils/createTest" {
    import { TestFunction, ValidatorInternalTest, ValidatorTestResult, ValidatorInternalTestResult } from "types/index";
    /**
     * Return a test default object
     * @param  {string}  options.name      The name of the test
     * @param  {function}  options.func    The function to call
     * @param  {object}  options.args      The arguments to pass to the function
     * @return {object}                    The final test object
     */
    export function createTest({ type, func, args, }: {
        type: string;
        func?: TestFunction;
        args?: any;
    }): ValidatorInternalTest;
    export function createTestResult(test: ValidatorInternalTest, valid: boolean, result?: ValidatorInternalTestResult): ValidatorTestResult;
}
declare module "validator" {
    import { ValidatorConfirm, ValidatorOptions, ValidatorSchema, AltheiaInstance, ValidatorErrorRaw, ValidatorErrorFormatted, ValidatorTestResult } from "types/index";
    /**
     * Validator class
     * new Validator({ foo: 'bar' });
     */
    class Validator {
        inst: AltheiaInstance;
        isValidator: number;
        validated: boolean;
        _schema: any;
        _body: any;
        _errors: ValidatorErrorFormatted[];
        _errorsRaw: ValidatorErrorRaw[];
        _confirm: ValidatorConfirm[];
        _options: ValidatorOptions;
        /**
         * Constructor
         * @param  {object} schema
         * @param  {object} inst   An Altheia instance
         */
        constructor(schema: ValidatorSchema, inst: AltheiaInstance);
        /**
         * Clone a validator
         * @return {Validator}
         */
        clone(): Validator;
        /**
         * Assign the body to validate
         *
         * @param  {object} body
         * @return {this}
         */
        body(body: any): this;
        /**
         * Declare the schema that describe the body()
         *
         * @param  {object} schema
         * @return {this}
         */
        schema(schema: ValidatorSchema): this;
        /**
         * Declare options to change the defaults behaviour of the Validator
         *
         * @param  {object} options
         * @return {this}
         */
        options(options: ValidatorOptions): this;
        /**
         * Format any Error Array returned by a test
         * @param  {object} error
         * @param  {string} label
         * @return {ValidatorErrorFormatted} Formatted error
         */
        formatError(error: ValidatorTestResult, label: string): ValidatorErrorFormatted;
        private flatten;
        /**
         * Validate the body against the schema
         *
         * @param  {function} callback
         * @return {false | ValidatorErrorFormatted[]} Test resutls
         */
        validate(callback?: (value: false | ValidatorErrorFormatted[]) => void): Promise<false | ValidatorErrorFormatted[]>;
        /**
         * Force the confirmation of one field by an other one
         *
         * @param  {string} initial    The first key
         * @param  {string} comparison The second key
         * @return {this}
         */
        confirm(initial: string, comparison: string): this;
    }
    export default Validator;
}
declare module "string" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * String class
     */
    export class TypeString extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force a string to be not empty
         *
         * @return {this}
         */
        noEmpty(): this;
        /**
         * Force a string to be equal or more to the value passed.
         *
         * @param  {number} min
         * @return {this}
         */
        min(min: number): this;
        /**
         * Force a string to be equal or less to the value passed.
         *
         * @param  {number} max
         * @return {this}
         */
        max(max: number): this;
        /**
         * Force a string to match the regex passed.
         *
         * @param  {Regex} regex
         * @return {this}
         */
        pattern(regex: RegExp): this;
        /**
         * Force a string to be equal to one of the value passed in the set.
         *
         * @param  {...string} obj
         * @return {this}
         */
        in(...obj: string[]): this;
        /**
         * Force a string to be different to all of the value passed in the set.
         *
         * @param  {...string} obj
         * @return {this}
         */
        not(...obj: string[]): this;
        /**
         * Force a string to be a valid email (contain an @).
         *
         * @return {this}
         */
        email(): this;
        /**
         * Force a string to be fully in lowercase.
         *
         * @return {this}
         */
        lowercase(): this;
        /**
         * Force a string to be fully in uppercase.
         *
         * @return {this}
         */
        uppercase(): this;
    }
    const def: {
        Class: typeof TypeString;
        messages: LangList;
    };
    export default def;
}
declare module "array" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * Array class
     */
    export class TypeArray extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force an array to contains at least a number of items equal to the value passed.
         *
         * @param  {number} min
         * @return {this}
         */
        min(min: number): this;
        /**
         * Force an array to contains at most a number of items equal to the value passed.
         *
         * @param  {number} max
         * @return {this}
         */
        max(max: number): this;
        /**
         * Force an array to have only the values passed in the set
         *
         * @param  {...array} array
         * @return {this}
         */
        in(...array: any[]): this;
        /**
         * Force an array not to have the values passed in the set
         *
         * @param  {...array} array
         * @return {this}
         */
        not(...array: any[]): this;
        /**
         * Force an array to only have each item once
         *
         * @return {this}
         */
        unique(): this;
        /**
         * Force all array's items to match one of the template
         *
         * @param  {...Validator} templates
         * @return {this}
         */
        oneOf(...templates: TypeBase[]): this;
    }
    const def: {
        Class: typeof TypeArray;
        messages: LangList;
    };
    export default def;
}
declare module "boolean" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * Boolean class
     */
    export class TypeBoolean extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Try to cast value
         *
         * @param {mixed} value
         * @return {boolean|mixed}
         */
        _cast(value: any): boolean | any;
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force a boolean to equal true
         *
         * @return {this}
         */
        true(): this;
        /**
         * Force a boolean to equal false
         *
         * @return {this}
         */
        false(): this;
    }
    const def: {
        Class: typeof TypeBoolean;
        messages: LangList;
    };
    export default def;
}
declare module "date" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * Date class
     */
    export class TypeDate extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Try to cast value
         *
         * @param {mixed} value
         * @return {Date|mixed}
         */
        _cast(value: any): Date | any;
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force a date to be a valid ISO-8601.
         *
         * @return {this}
         */
        iso(): this;
        /**
         * Force a date to be a at least or bigger than value passed
         *
         * @param  {Date} min
         * @return {this}
         */
        min(min: Date): this;
        /**
         * Force a date to be less or equal than value passed
         *
         * @param  {Date} max
         * @return {this}
         */
        max(max: Date): this;
    }
    const def: {
        Class: typeof TypeDate;
        messages: LangList;
    };
    export default def;
}
declare module "function" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * Function class
     */
    export class TypeFunc extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
    }
    const def: {
        Class: typeof TypeFunc;
        messages: LangList;
    };
    export default def;
}
declare module "utils/luhn" {
    const _default_1: (ccNum: string) => boolean | 0;
    export default _default_1;
}
declare module "internet" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * Internet class
     */
    export class TypeInternet extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force a string to be a valid url (RFC)
         *
         * @return {this}
         */
        url(): this;
        /**
         * Force a string to be a valid hostname (RFC)
         *
         * @return {this}
         */
        hostname(): this;
        /**
         * Force a string to be a valid hex (a-f0-9)
         *
         * @return {this}
         */
        hex(): this;
        /**
         * Force a string to be a valid credit card (using Luhn's algorithm)
         *
         * @return {this}
         */
        creditCard(): this;
        /**
         * Force a string to be a valid uuid version 4
         *
         * @return {this}
         */
        uuidv4(): this;
        /**
         * Force a string to be a valid ipv4 or ipv6
         *
         * @return {this}
         */
        ip(): this;
        /**
         * Force a string to be a valid ipv4
         *
         * @return {this}
         */
        ipv4(): this;
        /**
         * Force a string to be a valid ipv6
         *
         * @return {this}
         */
        ipv6(): this;
    }
    const def: {
        Class: typeof TypeInternet;
        messages: LangList;
    };
    export default def;
}
declare module "number" {
    import TypeBase from "base";
    import { LangList } from "types/index";
    export const messages: LangList;
    /**
     * Number class
     */
    export class TypeNumber extends TypeBase {
        /**
         * Constructor
         */
        constructor();
        /**
         * Try to cast value
         *
         * @param {mixed} value
         * @return {Number|mixed}
         */
        _cast(value: any): number | any;
        /**
         * Test to validate the type of the value
         *
         * @return {this}
         */
        typeof(): this;
        /**
         * Force a number to be equal or more to the value passed.
         *
         * @param  {number} min
         * @return {this}
         */
        min(min: number): this;
        /**
         * Force a number to be equal or less to the value passed.
         *
         * @param  {number} max
         * @return {this}
         */
        max(max: number): this;
        /**
         * Force a number to be an integer.
         *
         * @return {this}
         */
        integer(): this;
        /**
         * Force a number to be unsigned.
         *
         * @return {this}
         */
        unsigned(): this;
        /**
         * Force a number to be greater than 0.
         *
         * @return {this}
         */
        positive(): this;
        /**
         * Force a number to be lesser than 0.
         *
         * @return {this}
         */
        negative(): this;
        /**
         * Force a number to be equal to one of the value passed in the set.
         *
         * @param  {...number} obj
         * @return {this}
         */
        in(...obj: number[]): this;
        /**
         * Force a number to be different to all of the value passed in the set.
         *
         * @param  {...number} obj
         * @return {this}
         */
        not(...obj: number[]): this;
    }
    const def: {
        Class: typeof TypeNumber;
        messages: LangList;
    };
    export default def;
}
declare module "types/index" {
    import TypeBase from "base";
    import Validator from "validator";
    import { TypeString } from "string";
    import { TypeAny } from "any";
    import { TypeArray } from "array";
    import { TypeBoolean } from "boolean";
    import { TypeDate } from "date";
    import { TypeFunc } from "function";
    import { TypeInternet } from "internet";
    import { TypeNumber } from "number";
    import { TypeObject } from "object";
    export interface ValidatorTemplates {
        [k: string]: TypeBase;
    }
    export interface AltheiaInstance {
        [k: string]: any;
        (schema: ValidatorSchema): Validator;
        langList: LangList;
        templates: ValidatorTemplates;
        Base: typeof TypeBase;
        instance: (lang?: LangList) => AltheiaInstance;
        use: (plugin: {
            Class: BaseConstructor;
            messages?: LangList;
        }) => this;
        lang: (key: LangList | string, tpl?: LangFunction) => void;
        template: (name: string, schema: TypeBase) => void;
        is: (name: string) => TypeBase;
        formatError: ({ type, args, result }: ValidatorTestResult, label?: string, position?: number) => ValidatorErrorFormatted;
        any: () => TypeAny;
        array: () => TypeArray;
        boolean: () => TypeBoolean;
        date: () => TypeDate;
        func: () => TypeFunc;
        internet: () => TypeInternet;
        number: () => TypeNumber;
        object: () => TypeObject;
        string: () => TypeString;
    }
    export interface BaseConstructor {
        new (instance?: AltheiaInstance): TypeBase;
    }
    export type LangFunction = (name: string, args?: any, result?: any) => string;
    export interface LangList {
        [k: string]: LangFunction;
    }
    export interface ValidatorOptions {
        required?: boolean;
        unknown?: boolean;
        flatten?: boolean;
    }
    export interface ValidatorConfirm {
        initial: string;
        comparison: string;
    }
    export interface ValidatorSchema {
        [k: string]: TypeBase;
    }
    export interface ValidatorTestResult {
        valid: boolean;
        type: string;
        func: TestFunction;
        args: any;
        result?: ValidatorInternalTestResult;
    }
    export interface ValidatorErrorRaw {
        test: ValidatorTestResult;
        label: string;
        position?: number;
    }
    export interface ValidatorErrorFormatted {
        label: string;
        type: string;
        message: string;
        position?: number;
        errors?: ValidatorErrorFormatted[];
    }
    export type TestFunction = (params: any) => Promise<boolean | ValidatorInternalTestResult> | boolean | ValidatorInternalTestResult;
    export interface ValidatorInternalTest {
        type: string;
        func: TestFunction;
        args: any;
    }
    export interface ValidatorInternalTestResult {
        valid: boolean;
        error: string;
        overrideWith?: ValidatorTestResult;
        errors?: ValidatorErrorRaw[];
    }
}
declare module "base" {
    import { ValidatorInternalTest, TestFunction, AltheiaInstance, ValidatorTestResult } from "types/index";
    import { createTest, createTestResult } from "utils/createTest";
    /**
     * All type inherit this Class
     */
    class TypeBase {
        inst: AltheiaInstance;
        tests: (() => ValidatorInternalTest)[];
        name?: string;
        _required: boolean;
        _needCast: boolean;
        _noEmpty?: boolean;
        /**
         * Constructor
         *
         * @param  {Altheia} inst
         */
        constructor(inst?: AltheiaInstance);
        /**
         * Clone this class
         *
         * @return {TypeBase}
         */
        clone<T extends this>(): T;
        /**
         * Add a test
         *
         * @param  {string} name
         * @param  {function} func
         * @param  {Object} args
         * @return {undefined}
         */
        test(name: string, func: TestFunction, args?: {}): void;
        createTest: typeof createTest;
        createTestResult: typeof createTestResult;
        /**
         * Validate a value based on all tests added
         * @param  {mixed}   toTest
         * @param  {Function} callback
         * @return {object}
         */
        validate(toTest: any, callback?: (value: false | ValidatorTestResult) => void): Promise<false | ValidatorTestResult>;
        /**
         * Force the value to be non-null
         *
         * @return {this}
         */
        required(): this;
        /**
         * Force the value to be casted before any check
         *
         * @return {this}
         */
        cast(): this;
        /**
         * Custom validator
         *
         * @param  {string}   name
         * @param  {Function} callback
         * @param  {Function}   message
         * @return {this}
         */
        custom(name: string, callback: TestFunction): this;
        /**
         * Presence validation
         *
         * @param  {mixed} toTest
         * @return {boolean}
         */
        presence(toTest: any): boolean;
        /**
         * If validation
         *
         * @param  {function} options.test
         * @param  {function} options.then
         * @param  {function} options.otherwise
         * @return {this}
         */
        if<T extends this>({ test, then, otherwise, }: {
            test: (chain: T) => TypeBase;
            then: (chain: T) => TypeBase;
            otherwise: (chain: T) => TypeBase;
        }): this;
        private testToTestResult;
    }
    export default TypeBase;
}
declare module "any" {
    import Base from "base";
    /**
     * Any is just a empty class to allow chaining
     */
    export class TypeAny extends Base {
        /**
         * Constructor
         */
        constructor();
    }
    const def: {
        Class: typeof TypeAny;
    };
    export default def;
}
declare module "lang" {
    import { LangList } from "types/index";
    const messages: LangList;
    export default messages;
}
declare module "index" {
    import { LangList, AltheiaInstance } from "types/index";
    export const Instance: (lang?: LangList | undefined) => AltheiaInstance;
    const _default_2: AltheiaInstance;
    export default _default_2;
}
